import sys
import unittest
from os.path import abspath, dirname

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

from glossary_v2_errors_test import TestGlossaryErrorsBase

from pyglossary.glossary_v2 import ConvertArgs, Error, Glossary

__all__ = ["TestGlossaryStarDictBase"]


class TestGlossaryStarDictBase(TestGlossaryErrorsBase):
	def convert_txt_stardict(  # noqa: PLR0913
		self,
		fname,
		sdDirName,
		syn=True,
		dictzip=False,
		config=None,
		writeOptions=None,
		info=None,
		**convertArgs,
	):
		binExtList = ["idx", "dict"]
		if syn:
			binExtList.append("syn")

		inputFilename = self.downloadFile(f"{fname}.txt")
		outputFilename = self.newTempFilePath(f"{fname}.ifo")
		otherFiles = {ext: self.newTempFilePath(f"{fname}.{ext}") for ext in binExtList}

		glos = self.glos = Glossary()
		if info:
			for key, value in info.items():
				glos.setInfo(key, value)

		if config is not None:
			glos.config = config

		if writeOptions is None:
			writeOptions = {}
		writeOptions["dictzip"] = dictzip

		result = glos.convert(
			ConvertArgs(
				inputFilename=inputFilename,
				outputFilename=outputFilename,
				writeOptions=writeOptions,
				**convertArgs,
			)
		)
		self.assertEqual(outputFilename, result)

		self.compareTextFiles(
			outputFilename,
			self.downloadFile(f"{sdDirName}/{fname}.ifo"),
		)

		for ext in binExtList:
			self.compareBinaryFiles(
				otherFiles[ext],
				self.downloadFile(f"{sdDirName}/{fname}.{ext}"),
			)

	def convert_txt_stardict_zip(  # noqa: PLR0913
		self,
		fname,
		sha1sumDict,
		dictzip=False,
		config=None,
		**convertArgs,
	):
		inputFilename = self.downloadFile(f"{fname}.txt")
		outputFilename = self.newTempFilePath(f"{fname}.zip")

		glos = self.glos = Glossary()

		if config is not None:
			glos.config = config

		result = glos.convert(
			ConvertArgs(
				inputFilename=inputFilename,
				outputFilename=outputFilename,
				outputFormat="Stardict",
				writeOptions={
					"dictzip": dictzip,
				},
				**convertArgs,
			)
		)
		self.assertEqual(outputFilename, result)

		self.checkZipFileSha1sum(
			outputFilename,
			sha1sumDict=sha1sumDict,
		)

	def convert_stardict_txt(
		self,
		inputFname: str,
		inputDirName: str,
		outputFname: str,
		testId: str,
		syn=True,
		**convertArgs,
	):
		binExtList = ["idx", "dict"]
		if syn:
			binExtList.append("syn")
		for ext in binExtList:
			self.downloadFile(f"{inputDirName}/{inputFname}.{ext}")

		inputFilename = self.downloadFile(f"{inputDirName}/{inputFname}.ifo")
		outputFilename = self.newTempFilePath(
			f"{inputFname}-{testId}.txt",
		)
		expectedFilename = self.downloadFile(f"{outputFname}.txt")
		glos = self.glos = Glossary()

		result = glos.convert(
			ConvertArgs(
				inputFilename=inputFilename,
				outputFilename=outputFilename,
				**convertArgs,
			)
		)
		self.assertEqual(outputFilename, result)

		self.compareTextFiles(outputFilename, expectedFilename)


class TestGlossaryStarDict(TestGlossaryStarDictBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryErrorsBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"004-bar.sd/004-bar.dict": "9ea397f8",
				"004-bar.sd/004-bar.idx": "cf9440cf",
				"004-bar.sd/004-bar.ifo": "ada870e4",
				"004-bar.sd/004-bar.syn": "286b17bf",
				"100-en-de-v4-sd-v2/100-en-de-v4.dict": "5a97476f",
				"100-en-de-v4-sd-v2/100-en-de-v4.idx": "a99f29d2",
				"100-en-de-v4-sd-v2/100-en-de-v4.ifo": "2120708c",
				"100-en-fa-sd-v2/100-en-fa.dict": "223a0d1d",
				"100-en-fa-sd-v2/100-en-fa.idx": "6df43378",
				"100-en-fa-sd-v2/100-en-fa.ifo": "bb916827",
				"100-en-fa-sd-v2/100-en-fa.syn": "1160fa0b",
				"100-en-fa-sd-v2.txt": "0b8b2ac0",
				"100-en-fa-sd-nostt/100-en-fa.dict": "efdfaf68",
				"100-en-fa-sd-nostt/100-en-fa.idx": "dbfbce1e",
				"100-en-fa-sd-nostt/100-en-fa.ifo": "77d3b180",
				"100-en-fa-sd-nostt/100-en-fa.syn": "1160fa0b",
				# FIXME: remove empty description line from 100-en-fa.ifo
				# stardict-mixed-types-1.ifo, "stardict-mixed-types-2.ifo
				"100-ja-en.sd/100-ja-en.dict": "39715f01",
				"100-ja-en.sd/100-ja-en.idx": "adf0e552",
				"100-ja-en.sd/100-ja-en.ifo": "b01e368c",
				"100-ja-en.sd/100-ja-en.syn": "76e6df95",
				"300-ru-en.txt": "77cfee2f",
				"300-ru-en.sd/300-ru-en.dict": "8be7fa4c",
				"300-ru-en.sd/300-ru-en.idx": "1cd30f1a",
				"300-ru-en.sd/300-ru-en.ifo": "0b135812",
				"300-ru-en.sd/300-ru-en.syn": "87ee3372",
				"stardict-mixed-types-2.sd/stardict-mixed-types-2.dict": "2e43237a",
				"stardict-mixed-types-2.sd/stardict-mixed-types-2.idx": "65a1f9fc",
				"stardict-mixed-types-2.sd/stardict-mixed-types-2.ifo": "e1063b84",
				"stardict-mixed-types-2.sd.txt": "94de4bc6",
				"002-plain-html.txt": "75484314",
				"002-plain-html.sd/002-plain-html.dict": "2e9d20d8",
				"002-plain-html.sd/002-plain-html.idx": "3956ad72",
				"002-plain-html.sd/002-plain-html.ifo": "1991f125",
				"004-plain-html-alts.txt": "505d4675",
				"004-plain-html-alts.sd/004-plain-html-alts.dict": "889f11f8",
				"004-plain-html-alts.sd/004-plain-html-alts.idx": "edbe368d",
				"004-plain-html-alts.sd/004-plain-html-alts.ifo": "b9b92fa3",
				"004-plain-html-alts.sd/004-plain-html-alts.syn": "c07f7111",
			},
		)

	def test_convert_txt_stardict_0(self):
		self.convert_txt_stardict(
			"100-en-fa",
			"100-en-fa-sd-v2",
			config={"auto_sqlite": True},
			direct=True,
		)

	def test_convert_txt_stardict_1(self):
		for sqlite in (None, False, True):
			self.convert_txt_stardict(
				"100-en-fa",
				"100-en-fa-sd-v2",
				sqlite=sqlite,
			)

	def test_convert_txt_stardict_0_sametypeseq1(self):
		self.convert_txt_stardict(
			"100-en-fa",
			"100-en-fa-sd-nostt",
			writeOptions={
				"sametypesequence": "-",
			},
		)

	def test_convert_txt_stardict_0_sametypeseq2(self):
		self.convert_txt_stardict(
			"100-en-fa",
			"100-en-fa-sd-v2",
			writeOptions={
				"sametypesequence": None,  # same as "" means auto
			},
		)

	def test_convert_txt_stardict_1_zip(self):
		sha1sumDict = {
			"100-en-fa.dict": "1e462e829f9e2bf854ceac2ef8bc55911460c79e",
			"100-en-fa.idx": "943005945b35abf3a3e7b80375c76daa87e810f0",
			"100-en-fa.ifo": "bf12a932385f54dfcf5ab023d89a8dbd7091e60f",
			"100-en-fa.syn": "fcefc76628fed18b84b9aa83cd7139721b488545",
		}
		for sqlite in (None, False, True):
			self.convert_txt_stardict_zip(
				"100-en-fa",
				sha1sumDict=sha1sumDict,
				sqlite=sqlite,
			)

	def test_convert_txt_stardict_2(self):
		for sqlite in (None, False, True):
			self.convert_txt_stardict(
				"004-bar",
				"004-bar.sd",
				sqlite=sqlite,
			)

	def test_convert_txt_stardict_3(self):
		for sqlite in (None, False, True):
			self.convert_txt_stardict(
				"100-en-de-v4",
				"100-en-de-v4-sd-v2",
				syn=False,
				sqlite=sqlite,
			)

	def test_convert_txt_stardict_4(self):
		for sqlite in (None, False, True):
			self.convert_txt_stardict(
				"100-ja-en",
				"100-ja-en.sd",
				syn=True,
				sqlite=sqlite,
			)

	def test_convert_txt_stardict_5(self):
		for sqlite in (None, False, True):
			self.convert_txt_stardict(
				"300-ru-en",
				"300-ru-en.sd",
				syn=True,
				sqlite=sqlite,
			)

	def test_convert_txt_stardict_sqlite_no_alts(self):
		self.convert_txt_stardict(
			"100-en-fa",
			"100-en-fa-sd-v2",
			config={"enable_alts": False},
			sqlite=True,
		)
		self.assertLogWarning(
			"SQLite mode only works with enable_alts=True, force-enabling it.",
		)

	def test_convert_stardict_txt_1(self):
		self.convert_stardict_txt(
			"100-en-fa",
			"100-en-fa-sd-v2",
			"100-en-fa-sd-v2",
			"1",
		)

	def test_convert_stardict_txt_mixed_types_1(self):
		self.convert_stardict_txt(
			"stardict-mixed-types-2",
			"stardict-mixed-types-2.sd",
			"stardict-mixed-types-2.sd",
			"mixed-types-1",
			syn=False,
		)

	def test_convert_stardict_txt_mixed_types_2(self):
		self.convert_stardict_txt(
			"stardict-mixed-types-2",
			"stardict-mixed-types-2.sd",
			"stardict-mixed-types-2.sd",
			"mixed-types-1",
			syn=False,
			readOptions={"xdxf_to_html": False},
		)

	def test_convert_txt_stardict_general_1(self):
		self.convert_txt_stardict(
			"002-plain-html",
			"002-plain-html.sd",
			syn=False,
		)

	def test_convert_txt_stardict_general_2(self):
		self.convert_txt_stardict(
			"004-plain-html-alts",
			"004-plain-html-alts.sd",
			syn=True,
		)


class TestGlossaryErrorsStarDict(TestGlossaryErrorsBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryErrorsBase.__init__(self, *args, **kwargs)

	def test_convert_from_stardict_invalid_sametypesequence(self):
		fname = "foobar"
		inputFilename = self.newTempFilePath(f"{fname}.ifo")
		outputFilename = self.newTempFilePath(f"{fname}.txt")

		with open(inputFilename, mode="w", encoding="utf-8") as file:
			file.write(
				"""StarDict's dict ifo file
version=3.0.0
bookname=Test
wordcount=123
idxfilesize=1234
sametypesequence=abcd
""",
			)

		glos = self.glos = Glossary()

		with self.assertRaisesRegex(
			Error,
			"Invalid sametypesequence = 'abcd'",
		):
			glos.convert(
				ConvertArgs(
					inputFilename=inputFilename,
					outputFilename=outputFilename,
				)
			)

		# self.assertLogCritical(f"Reading file {relpath(inputFilename)!r} failed.")


if __name__ == "__main__":
	unittest.main()



from __future__ import annotations

from typing import TYPE_CHECKING

from gi.repository import Gtk as gtk

from .log_handler import GtkSingleTextviewLogHandler
from .utils import rgba_parse

if TYPE_CHECKING:
	import logging

	from .log_handler import MainWinType

__all__ = ["ConvertConsole"]


class ConvertConsole(gtk.ScrolledWindow):
	def __init__(
		self,
		mainWin: MainWinType,
		lightTheme: bool = False,
	) -> None:
		gtk.ScrolledWindow.__init__(self)
		self.set_policy(gtk.PolicyType.AUTOMATIC, gtk.PolicyType.AUTOMATIC)
		self._textview = textview = gtk.TextView()
		self.set_child(textview)
		##########
		textview.get_style_context().add_class("console")
		self._handler = handler = GtkSingleTextviewLogHandler(mainWin, textview)
		###
		normalColor = rgba_parse("black" if lightTheme else "white")
		errorColor = rgba_parse("red")
		warningColor = rgba_parse("hsl(30, 100%, 50%)" if lightTheme else "yellow")
		###
		handler.setColor("CRITICAL", errorColor)
		handler.setColor("ERROR", errorColor)
		handler.setColor("WARNING", warningColor)
		handler.setColor("INFO", normalColor)
		handler.setColor("DEBUG", normalColor)
		handler.setColor("TRACE", normalColor)
		###
		textview.get_buffer().set_text("Output & Error Console:\n")
		textview.set_editable(False)

	@property
	def handler(self) -> logging.Handler:
		return self._handler

	def set_text(self, text: str) -> None:
		self._textview.get_buffer().set_text(text)



import logging
import sys
import unittest
from os.path import abspath, dirname

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

from glossary_v2_errors_test import TestGlossaryErrors

from pyglossary.glossary_v2 import ConvertArgs, Error, Glossary, ReadError


class TestGlossarySecurity(TestGlossaryErrors):
	def __init__(self, *args, **kwargs):
		TestGlossaryErrors.__init__(self, *args, **kwargs)
		self.mockLog.setLevel(logging.INFO)

	def test_convert_1(self):
		glos = Glossary()
		with self.assertRaisesRegex(Error, "Unable to detect output format!"):
			glos.convert(
				ConvertArgs(
					inputFilename="os.system('abcd')",
					outputFilename="os.system('abcd -l')",
				)
			)

	def test_convert_2(self):
		glos = Glossary()
		with self.assertRaisesRegex(Error, "Unable to detect output format!"):
			glos.convert(
				ConvertArgs(
					inputFilename="os.system('abcd');test.txt",
					outputFilename="os.system('abcd -l')",
				)
			)

	def test_convert_3(self):
		glos = Glossary()
		with self.assertRaisesRegex(ReadError, "No such file or directory: "):
			glos.convert(
				ConvertArgs(
					inputFilename="os.system('abcd');test.txt",
					outputFilename="os.system('abcd -l');test.csv",
				)
			)

	def test_convert_4(self):
		glos = Glossary()
		with self.assertRaisesRegex(Error, "Unable to detect output format!"):
			glos.convert(
				ConvertArgs(
					inputFilename="test.txt\nos.system('abcd')",
					outputFilename="test.csv\nos.system('abcd -l')",
				)
			)


if __name__ == "__main__":
	unittest.main()



import sys
import unittest
from os.path import abspath, dirname

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

from pyglossary.apple_utils import substituteAppleCSS


class Test_substituteAppleCSS(unittest.TestCase):
	def test_remove(self):
		css = b""".test { -webkit-text-combine: horizontal; color: black }
.test2 {
	-apple-color-filter: none;
}"""
		fixed_expected = b""".test {color: black }
.test2 {
}"""
		fixed_actual = substituteAppleCSS(css)
		self.assertEqual(fixed_actual, fixed_expected)

	def test_1(self):
		css = b"""html.apple_display-separateview
{
	-webkit-column-width: 25em;
	-webkit-column-rule-color: LightGrey;
	-webkit-column-rule-style: solid;
	-webkit-column-rule-width: 1px;
}

span.sn
{
	-webkit-text-combine: horizontal;
	vertical-align: -6%;
}
"""
		fixed_expected = b"""html.apple_display-separateview
{
	column-width: 25em;
	column-rule-color: LightGrey;
	column-rule-style: solid;
	column-rule-width: 1px;
}

span.sn
{
vertical-align: -6%;
}
"""
		fixed_actual = substituteAppleCSS(css)
		self.assertEqual(fixed_actual, fixed_expected)


if __name__ == "__main__":
	unittest.main()



import os
import unittest

from glossary_v2_test import TestGlossaryBase


class TestGlossaryDictunformat(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"100-en-fa-2.dictunformat": "03a13c1a",
				"100-en-fa-2.dictunformat.txt": "c88207ec",
			},
		)
		os.environ["CALC_FILE_SIZE"] = "1"

	def convert_dictunformat_txt(self, fname, fname2, **convertArgs):
		self.convert(
			f"{fname}.dictunformat",
			f"{fname}-tmp.txt",
			compareText=f"{fname2}.txt",
			**convertArgs,
		)

	def test_convert_dictunformat_txt_1(self):
		self.convert_dictunformat_txt(
			"100-en-fa-2",
			"100-en-fa-2.dictunformat",
		)


if __name__ == "__main__":
	unittest.main()



# -*- coding: utf-8 -*-

from __future__ import annotations

from .text_utils import replacePostSpaceChar

__all__ = ["faEditStr"]


def faEditStr(st: str) -> str:
	return replacePostSpaceChar(
		st.replace("ي", "ی")
		.replace("ك", "ک")
		.replace("ۂ", "هٔ")
		.replace("\u06c1", "\u0647"),
		"،",
	)



from __future__ import annotations

import io
import logging
import os
import random
import sys
import tempfile
import unicodedata
import unittest
from os.path import abspath, dirname
from typing import cast

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

import icu

from pyglossary import slob
from pyglossary.core_test import MockLogHandler

mockLog = MockLogHandler()
log = logging.getLogger("pyglossary")
log.addHandler(mockLog)


class StructReaderWriter(slob.StructWriter):
	def __init__(
		self,
		file: io.BufferedWriter,
		reader: slob.StructReader,
		encoding: str | None = None,
	) -> None:
		super().__init__(
			file=file,
			encoding=encoding,
		)
		self._reader = reader

	def tell(self) -> int:
		return self._file.tell()

	def write(self, data: bytes) -> int:
		return self._file.write(data)

	def read_byte(self) -> int:
		return self._reader.read_byte()

	def read_tiny_text(self) -> str:
		return self._reader.read_tiny_text()


class TagNotFound(Exception):
	pass


def set_tag_value(filename: str, name: str, value: str) -> None:
	with slob.fopen(filename, "rb+") as file:
		file.seek(len(slob.MAGIC) + 16)
		encoding = slob.read_byte_string(file, slob.U_CHAR).decode(slob.UTF8)
		if slob.encodings.search_function(encoding) is None:
			raise slob.UnknownEncoding(encoding)
		reader = StructReaderWriter(
			file=file,
			reader=slob.StructReader(file, encoding=encoding),
			encoding=encoding,
		)
		reader.read_tiny_text()
		tag_count = reader.read_byte()
		for _ in range(tag_count):
			key = reader.read_tiny_text()
			if key == name:
				reader.write_tiny_text(value, editable=True)
				return
			reader.read_tiny_text()
	raise TagNotFound(name)


class BaseTest(unittest.TestCase):
	def setUp(self):
		self.tmpdir = tempfile.TemporaryDirectory(prefix="test")
		self._writers = []

	def tearDown(self):
		for w in self._writers:
			w.close()
		self.tmpdir.cleanup()

	def _observer(self, event: slob.WriterEvent):
		log.info(f"slob: {event.name}{': ' + event.data if event.data else ''}")

	def create(self, *args, observer=None, **kwargs):
		if observer is None:
			observer = self._observer
		w = slob.Writer(*args, observer=observer, **kwargs)
		self._writers.append(w)
		return w


class TestReadWrite(BaseTest):
	def setUp(self):
		BaseTest.setUp(self)

		self.path = os.path.join(self.tmpdir.name, "test.slob")

		writer = self.create(self.path)

		self.tags = {
			"a": "abc",
			"bb": "xyz123",
			"ccc": "lkjlk",
		}
		for name, value in self.tags.items():
			writer.tag(name, value)

		self.tag2 = "bb", "xyz123"

		self.blob_encoding = "ascii"

		self.data = [
			(("c", "cc", "ccc"), slob.MIME_TEXT, "Hello C 1"),
			("a", slob.MIME_TEXT, "Hello A 12"),
			("z", slob.MIME_TEXT, "Hello Z 123"),
			("b", slob.MIME_TEXT, "Hello B 1234"),
			("d", slob.MIME_TEXT, "Hello D 12345"),
			("uuu", slob.MIME_HTML, "<html><body>Hello U!</body></html>"),
			((("yy", "frag1"),), slob.MIME_HTML, '<h1 name="frag1">Section 1</h1>'),
		]

		self.all_keys = []

		self.data_as_dict = {}

		for k, t, v in self.data:
			if isinstance(k, str):
				k = (k,)  # noqa: PLW2901
			for key in k:
				if isinstance(key, tuple):
					key, fragment = key  # noqa: PLW2901
				else:
					fragment = ""
				self.all_keys.append(key)
				self.data_as_dict[key] = (t, v, fragment)
			writer.add(v.encode(self.blob_encoding), *k, content_type=t)
		self.all_keys.sort()

		writer.finalize()
		self.w = writer

	def test_header(self):
		with slob.MultiFileReader(self.path) as f:
			header = slob.read_header(f)

		for key, value in self.tags.items():
			self.assertEqual(header.tags[key], value)

		self.assertEqual(self.w.encoding, slob.UTF8)
		self.assertEqual(header.encoding, self.w.encoding)

		self.assertEqual(header.compression, self.w.compression)

		for i, content_type in enumerate(header.content_types):
			self.assertEqual(self.w.content_types[content_type], i)

		self.assertEqual(header.blob_count, len(self.data))

	def test_content(self):
		with slob.open(self.path) as r:
			self.assertEqual(len(r), len(self.all_keys))
			self.assertRaises(IndexError, r.__getitem__, len(self.all_keys))
			for i, item in enumerate(r):
				self.assertEqual(item.key, self.all_keys[i])
				content_type, value, fragment = self.data_as_dict[item.key]
				self.assertEqual(item.content_type, content_type)
				self.assertEqual(item.content.decode(self.blob_encoding), value)
				self.assertEqual(item.fragment, fragment)


class TestSort(BaseTest):
	def setUp(self):
		BaseTest.setUp(self)

		self.path = os.path.join(self.tmpdir.name, "test.slob")

		writer = self.create(self.path)
		data = [
			"Ф, ф",
			"Ф ф",
			"Ф",
			"Э",
			"Е е",
			"г",
			"н",
			"ф",
			"а",
			"Ф, Ф",
			"е",
			"Е",
			"Ее",
			"ё",
			"Ё",
			"Её",
			"Е ё",
			"А",
			"э",
			"ы",
		]

		self.data_sorted = sorted(data, key=slob.sortkey(slob.IDENTICAL))

		for k in data:
			v = ";".join(unicodedata.name(c) for c in k)
			writer.add(v.encode("ascii"), k)

		writer.finalize()

		self.r = slob.open(self.path)

	def test_sort_order(self):
		for i in range(len(self.r)):
			self.assertEqual(self.r[i].key, self.data_sorted[i])

	def tearDown(self):
		self.r.close()
		BaseTest.tearDown(self)


class TestSortKey(BaseTest):
	def setUp(self):
		BaseTest.setUp(self)

		self.data = [
			"Ф, ф",
			"Ф ф",
			"Ф",
			"Э",
			"Е е",
			"г",
			"н",
			"ф",
			"а",
			"Ф, Ф",
			"е",
			"Е",
			"Ее",
			"ё",
			"Ё",
			"Её",
			"Е ё",
			"А",
			"э",
			"ы",
		]
		self.data_sorted = [
			"а",
			"А",
			"г",
			"е",
			"Е",
			"ё",
			"Ё",
			"Е е",
			"Ее",
			"Е ё",
			"Её",
			"н",
			"ф",
			"Ф",
			"Ф ф",
			"Ф, ф",
			"Ф, Ф",
			"ы",
			"э",
			"Э",
		]

	def test_sort_order(self):
		for locName in (
			# en_US_POSIX on Mac OS X
			# https://github.com/ilius/pyglossary/issues/458
			"en_US_POSIX",
			"en_US",
			"en_CA",
			"fa_IR.UTF-8",
		):
			icu.Locale.setDefault(icu.Locale(locName))
			slob.sortkey.cache_clear()
			data_sorted = sorted(self.data, key=slob.sortkey(slob.IDENTICAL))
			self.assertEqual(self.data_sorted, data_sorted)


class TestFind(BaseTest):
	def setUp(self):
		BaseTest.setUp(self)

		self.path = os.path.join(self.tmpdir.name, "test.slob")

		writer = self.create(self.path)
		data = [
			"Cc",
			"aA",
			"aa",
			"Aa",
			"Bb",
			"cc",
			"Äā",
			"ăÀ",
			"a\u00a0a",
			"a-a",
			"a\u2019a",
			"a\u2032a",
			"a,a",
			"a a",
		]

		for k in data:
			v = ";".join(unicodedata.name(c) for c in k)
			writer.add(v.encode("ascii"), k)

		writer.finalize()

		self.r = slob.open(self.path)

	def get(self, d, key):
		return [item.content.decode("ascii") for item in d[key]]

	def test_find_identical(self):
		d = self.r.as_dict(slob.IDENTICAL)
		self.assertEqual(
			self.get(d, "aa"),
			["LATIN SMALL LETTER A;LATIN SMALL LETTER A"],
		)
		self.assertEqual(
			self.get(d, "a-a"),
			["LATIN SMALL LETTER A;HYPHEN-MINUS;LATIN SMALL LETTER A"],
		)
		self.assertEqual(
			self.get(d, "aA"),
			["LATIN SMALL LETTER A;LATIN CAPITAL LETTER A"],
		)
		self.assertEqual(
			self.get(d, "Äā"),
			[
				"LATIN CAPITAL LETTER A WITH DIAERESIS;LATIN SMALL LETTER A WITH MACRON",
			],
		)
		self.assertEqual(
			self.get(d, "a a"),
			["LATIN SMALL LETTER A;SPACE;LATIN SMALL LETTER A"],
		)

	def test_find_quaternary(self):
		d = self.r.as_dict(slob.QUATERNARY)
		self.assertEqual(
			self.get(d, "a\u2032a"),
			["LATIN SMALL LETTER A;PRIME;LATIN SMALL LETTER A"],
		)
		self.assertEqual(
			self.get(d, "a a"),
			[
				"LATIN SMALL LETTER A;SPACE;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;NO-BREAK SPACE;LATIN SMALL LETTER A",
			],
		)

	def test_find_tertiary(self):
		d = self.r.as_dict(slob.TERTIARY)
		self.assertEqual(
			self.get(d, "aa"),
			[
				"LATIN SMALL LETTER A;SPACE;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;NO-BREAK SPACE;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;HYPHEN-MINUS;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;COMMA;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;RIGHT SINGLE QUOTATION MARK;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;PRIME;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;LATIN SMALL LETTER A",
			],
		)

	def test_find_secondary(self):
		d = self.r.as_dict(slob.SECONDARY)
		self.assertEqual(
			self.get(d, "aa"),
			[
				"LATIN SMALL LETTER A;SPACE;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;NO-BREAK SPACE;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;HYPHEN-MINUS;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;COMMA;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;RIGHT SINGLE QUOTATION MARK;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;PRIME;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;LATIN SMALL LETTER A",
				"LATIN SMALL LETTER A;LATIN CAPITAL LETTER A",
				"LATIN CAPITAL LETTER A;LATIN SMALL LETTER A",
			],
		)

	def test_find_primary(self):
		d = self.r.as_dict(slob.PRIMARY)

		expected = [
			"LATIN SMALL LETTER A;SPACE;LATIN SMALL LETTER A",
			"LATIN SMALL LETTER A;NO-BREAK SPACE;LATIN SMALL LETTER A",
			"LATIN SMALL LETTER A;HYPHEN-MINUS;LATIN SMALL LETTER A",
			"LATIN SMALL LETTER A;COMMA;LATIN SMALL LETTER A",
			"LATIN SMALL LETTER A;RIGHT SINGLE QUOTATION MARK;LATIN SMALL LETTER A",
			"LATIN SMALL LETTER A;PRIME;LATIN SMALL LETTER A",
			"LATIN SMALL LETTER A;LATIN SMALL LETTER A",
			"LATIN SMALL LETTER A;LATIN CAPITAL LETTER A",
			"LATIN CAPITAL LETTER A;LATIN SMALL LETTER A",
			"LATIN SMALL LETTER A WITH BREVE;LATIN CAPITAL LETTER A WITH GRAVE",
			"LATIN CAPITAL LETTER A WITH DIAERESIS;LATIN SMALL LETTER A WITH MACRON",
		]
		self.assertEqual(
			self.get(d, "aa"),
			expected,
		)

	def tearDown(self):
		self.r.close()
		BaseTest.tearDown(self)


class TestPrefixFind(BaseTest):
	def setUp(self):
		BaseTest.setUp(self)

		self.path = os.path.join(self.tmpdir.name, "test.slob")
		self.data = ["a", "ab", "abc", "abcd", "abcde"]
		writer = self.create(self.path)
		for k in self.data:
			writer.add(k.encode("ascii"), k)
		writer.finalize()

	def test(self):
		with slob.open(self.path) as r:
			for i, k in enumerate(self.data):
				d = r.as_dict(slob.IDENTICAL, len(k))
				self.assertEqual(
					[cast("slob.Blob", v).content.decode("ascii") for v in d[k]],
					self.data[i:],
				)


class TestAlias(BaseTest):
	def setUp(self):
		BaseTest.setUp(self)

		self.path = os.path.join(self.tmpdir.name, "test.slob")

	def test_alias(self):
		too_many_redirects = []
		target_not_found = []

		def observer(event):
			if event.name == "too_many_redirects":
				too_many_redirects.append(event.data)
			elif event.name == "alias_target_not_found":
				target_not_found.append(event.data)

		w = self.create(self.path, observer=observer)
		data = ["z", "b", "q", "a", "u", "g", "p", "n"]
		for k in data:
			v = ";".join(unicodedata.name(c) for c in k)
			w.add(v.encode("ascii"), k)

		w.add_alias("w", "u")
		w.add_alias("small u", "u")
		w.add_alias("y1", "y2")
		w.add_alias("y2", "y3")
		w.add_alias("y3", "z")
		w.add_alias("ZZZ", "YYY")

		w.add_alias("l3", "l1")
		w.add_alias("l1", "l2")
		w.add_alias("l2", "l3")

		w.add_alias("a1", ("a", "a-frag1"))
		w.add_alias("a2", "a1")
		w.add_alias("a3", ("a2", "a-frag2"))

		w.add_alias("g1", "g")
		w.add_alias("g2", ("g1", "g-frag1"))

		w.add_alias("n or p", "n")
		w.add_alias("n or p", "p")

		w.finalize()

		self.assertEqual(too_many_redirects, ["l1", "l2", "l3"])
		self.assertEqual(target_not_found, ["l2", "l3", "l1", "YYY"])

		with slob.open(self.path) as r:
			d = r.as_dict()

			def get(key):
				return [item.content.decode("ascii") for item in d[key]]

			self.assertEqual(get("w"), ["LATIN SMALL LETTER U"])
			self.assertEqual(get("small u"), ["LATIN SMALL LETTER U"])
			self.assertEqual(get("y1"), ["LATIN SMALL LETTER Z"])
			self.assertEqual(get("y2"), ["LATIN SMALL LETTER Z"])
			self.assertEqual(get("y3"), ["LATIN SMALL LETTER Z"])
			self.assertEqual(get("ZZZ"), [])
			self.assertEqual(get("l1"), [])
			self.assertEqual(get("l2"), [])
			self.assertEqual(get("l3"), [])

			self.assertEqual(len(list(d["n or p"])), 2)

			item_a1 = cast("slob.Blob", next(d["a1"]))
			self.assertEqual(item_a1.content, b"LATIN SMALL LETTER A")
			self.assertEqual(item_a1.fragment, "a-frag1")

			item_a2 = cast("slob.Blob", next(d["a2"]))
			self.assertEqual(item_a2.content, b"LATIN SMALL LETTER A")
			self.assertEqual(item_a2.fragment, "a-frag1")

			item_a3 = cast("slob.Blob", next(d["a3"]))
			self.assertEqual(item_a3.content, b"LATIN SMALL LETTER A")
			self.assertEqual(item_a3.fragment, "a-frag1")

			item_g1 = cast("slob.Blob", next(d["g1"]))
			self.assertEqual(item_g1.content, b"LATIN SMALL LETTER G")
			self.assertEqual(item_g1.fragment, "")

			item_g2 = cast("slob.Blob", next(d["g2"]))
			self.assertEqual(item_g2.content, b"LATIN SMALL LETTER G")
			self.assertEqual(item_g2.fragment, "g-frag1")


class TestBlobId(BaseTest):
	def test(self):
		max_i = 2**32 - 1
		max_j = 2**16 - 1
		i_values = [0, max_i] + [random.randint(1, max_i - 1) for _ in range(100)]
		j_values = [0, max_j] + [random.randint(1, max_j - 1) for _ in range(100)]
		for i in i_values:
			for j in j_values:
				self.assertEqual(slob.unmeld_ints(slob.meld_ints(i, j)), (i, j))


class TestMultiFileReader(BaseTest):
	def test_read_all(self):
		fnames = []
		for name in "abcdef":
			path = os.path.join(self.tmpdir.name, name)
			fnames.append(path)
			with slob.fopen(path, "wb") as f:
				f.write(name.encode(slob.UTF8))
		with slob.MultiFileReader(*fnames) as m:
			self.assertEqual(m.read().decode(slob.UTF8), "abcdef")

	def test_seek_and_read(self):
		def mkfile(basename, content):
			part = os.path.join(self.tmpdir.name, basename)
			with slob.fopen(part, "wb") as f:
				f.write(content)
			return part

		content = b"abc\nd\nefgh\nij"
		part1 = mkfile("1", content[:4])
		part2 = mkfile("2", content[4:5])
		part3 = mkfile("3", content[5:])

		with slob.MultiFileReader(part1, part2, part3) as m:
			self.assertEqual(m.size, len(content))
			m.seek(2)
			self.assertEqual(m.read(2), content[2:4])
			m.seek(1)
			self.assertEqual(m.read(len(content) - 2), content[1:-1])
			m.seek(-1, whence=io.SEEK_END)
			self.assertEqual(m.read(10), content[-1:])

			m.seek(4)
			m.seek(-2, whence=io.SEEK_CUR)
			self.assertEqual(m.read(3), content[2:5])


class TestFormatErrors(BaseTest):
	def test_wrong_file_type(self):
		name = os.path.join(self.tmpdir.name, "1")
		with slob.fopen(name, "wb") as f:
			f.write(b"123")
		self.assertRaises(slob.UnknownFileFormat, slob.open, name)

	def test_truncated_file(self):
		name = os.path.join(self.tmpdir.name, "1")

		writer = self.create(name)
		writer.add(b"123", "a")
		writer.add(b"234", "b")
		writer.finalize()

		with slob.fopen(name, "rb") as f:
			all_bytes = f.read()

		with slob.fopen(name, "wb") as f:
			f.write(all_bytes[:-1])

		self.assertRaises(slob.IncorrectFileSize, slob.open, name)

		with slob.fopen(name, "wb") as f:
			f.write(all_bytes)
			f.write(b"\n")

		self.assertRaises(slob.IncorrectFileSize, slob.open, name)


class TestTooLongText(BaseTest):
	def setUp(self):
		BaseTest.setUp(self)
		self.path = os.path.join(self.tmpdir.name, "test.slob")

	def test_too_long(self):
		rejected_keys = []
		rejected_aliases = []
		rejected_alias_targets = []
		rejected_tags = []
		rejected_content_types = []

		def observer(event):
			if event.name == "key_too_long":
				rejected_keys.append(event.data)
			elif event.name == "alias_too_long":
				rejected_aliases.append(event.data)
			elif event.name == "alias_target_too_long":
				rejected_alias_targets.append(event.data)
			elif event.name == "tag_name_too_long":
				rejected_tags.append(event.data)
			elif event.name == "content_type_too_long":
				rejected_content_types.append(event.data)

		long_tag_name = "t" * (slob.MAX_TINY_TEXT_LEN + 1)
		long_tag_value = "v" * (slob.MAX_TINY_TEXT_LEN + 1)
		long_content_type = "T" * (slob.MAX_TEXT_LEN + 1)
		long_key = "c" * (slob.MAX_TEXT_LEN + 1)
		long_frag = "d" * (slob.MAX_TINY_TEXT_LEN + 1)
		key_with_long_frag = ("d", long_frag)
		tag_with_long_name = (long_tag_name, "t3 value")
		tag_with_long_value = ("t1", long_tag_value)
		long_alias = "f" * (slob.MAX_TEXT_LEN + 1)
		alias_with_long_frag = ("i", long_frag)
		long_alias_target = long_key
		long_alias_target_frag = key_with_long_frag

		w = self.create(self.path, observer=observer)
		w.tag(*tag_with_long_value)
		w.tag("t2", "t2 value")
		w.tag(*tag_with_long_name)

		data = ["a", "b", long_key, key_with_long_frag]

		for k in data:
			v = k.encode("ascii") if isinstance(k, str) else "#".join(k).encode("ascii")
			w.add(v, k)

		w.add_alias("e", "a")
		w.add_alias(long_alias, "a")
		w.add_alias(alias_with_long_frag, "a")
		w.add_alias("g", long_alias_target)
		w.add_alias("h", long_alias_target_frag)

		w.add(b"Hello", "hello", content_type=long_content_type)
		w.finalize()

		self.assertEqual(
			rejected_keys,
			[long_key, key_with_long_frag],
		)
		self.assertEqual(
			rejected_aliases,
			[long_alias, alias_with_long_frag],
		)
		self.assertEqual(
			rejected_alias_targets,
			[long_alias_target, long_alias_target_frag],
		)
		self.assertEqual(
			rejected_tags,
			[tag_with_long_name],
		)
		self.assertEqual(
			rejected_content_types,
			[long_content_type],
		)

		with slob.open(self.path) as r:
			self.assertEqual(r.tags["t2"], "t2 value")
			self.assertNotIn(tag_with_long_name[0], r.tags)
			self.assertIn(tag_with_long_value[0], r.tags)
			self.assertEqual(r.tags[tag_with_long_value[0]], "")
			d = r.as_dict()
			self.assertIn("a", d)
			self.assertIn("b", d)
			self.assertNotIn(long_key, d)
			self.assertNotIn(key_with_long_frag[0], d)
			self.assertIn("e", d)
			self.assertNotIn(long_alias, d)
			self.assertNotIn("g", d)

		self.assertRaises(
			ValueError,
			set_tag_value,
			self.path,
			"t1",
			"ы" * 128,
		)


class TestEditTag(BaseTest):
	def setUp(self):
		BaseTest.setUp(self)
		self.path = os.path.join(self.tmpdir.name, "test.slob")
		writer = self.create(self.path)
		writer.tag("a", "123456")
		writer.tag("b", "654321")
		writer.finalize()

	def test_edit_existing_tag(self):
		with slob.open(self.path) as f:
			self.assertEqual(f.tags["a"], "123456")
			self.assertEqual(f.tags["b"], "654321")
		set_tag_value(self.path, "b", "efg")
		set_tag_value(self.path, "a", "xyz")
		with slob.open(self.path) as f:
			self.assertEqual(f.tags["a"], "xyz")
			self.assertEqual(f.tags["b"], "efg")

	def test_edit_nonexisting_tag(self):
		self.assertRaises(TagNotFound, set_tag_value, self.path, "z", "abc")


class TestBinItemNumberLimit(BaseTest):
	def setUp(self):
		BaseTest.setUp(self)
		self.path = os.path.join(self.tmpdir.name, "test.slob")

	def test_writing_more_then_max_number_of_bin_items(self):
		writer = self.create(self.path)
		for _ in range(slob.MAX_BIN_ITEM_COUNT + 2):
			writer.add(b"a", "a")
		self.assertEqual(writer.bin_count, 2)
		writer.finalize()


if __name__ == "__main__":
	unittest.main()



import unittest

from glossary_v2_test import TestGlossaryBase


class TestGlossarySlobRead(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"100-en-fa-res.slob": "0216d006",
				"100-en-fa-res-slob.txt": "c73100b3",
				"100-en-fa-res-slob-sort.txt": "8253fe96",
			},
		)

	def setUp(self):
		TestGlossaryBase.setUp(self)

	def convert_slob_txt(self, fname, fname2, resFiles, **convertArgs):
		resFilesPath = {
			resFileName: self.newTempFilePath(f"{fname}-2.txt_res/{resFileName}")
			for resFileName in resFiles
		}

		self.convert(
			f"{fname}.slob",
			f"{fname}-2.txt",
			compareText=f"{fname2}.txt",
			**convertArgs,
		)

		for resFileName in resFiles:
			fpath1 = self.downloadFile(f"res/{resFileName}")
			fpath2 = resFilesPath[resFileName]
			self.compareBinaryFiles(fpath1, fpath2)

	def test_convert_slob_txt_1(self):
		self.convert_slob_txt(
			"100-en-fa-res",
			"100-en-fa-res-slob",
			resFiles=[
				"stardict.png",
				"test.json",
			],
		)

	def test_convert_slob_txt_2(self):
		self.convert_slob_txt(
			"100-en-fa-res",
			"100-en-fa-res-slob",
			resFiles=[
				"stardict.png",
				"test.json",
			],
			direct=False,
		)

	def test_convert_slob_txt_3(self):
		self.convert_slob_txt(
			"100-en-fa-res",
			"100-en-fa-res-slob",
			resFiles=[
				"stardict.png",
				"test.json",
			],
			sqlite=True,
		)

	def test_convert_slob_txt_4(self):
		self.convert_slob_txt(
			"100-en-fa-res",
			"100-en-fa-res-slob-sort",
			resFiles=[
				"stardict.png",
				"test.json",
			],
			sort=True,
		)


if __name__ == "__main__":
	unittest.main()



import sys
import unittest
from os.path import abspath, dirname

rootDir = dirname(dirname(dirname(abspath(__file__))))
sys.path.insert(0, rootDir)

from glossary_test import TestGlossaryBase

from pyglossary.glossary import Glossary as GlossaryLegacy


class TestLegacyGlossaryCSV(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"100-en-de-v4.csv": "2890fb3e",
				"100-en-fa.csv": "eb8b0474",
				"100-en-fa-semicolon.csv": "b3f04599",
				"100-ja-en.csv": "7af18cf3",
			},
		)

	def convert_csv_txt_rw(self, fname, fname2, infoOverride=None):
		inputFilename = self.downloadFile(f"{fname}.csv")
		outputFilename = self.newTempFilePath(f"{fname}-2.txt")
		expectedFilename = self.downloadFile(f"{fname2}.txt")
		glos = self.glos = GlossaryLegacy()
		# using glos.convert will add "input_file_size" info key
		# perhaps add another optional argument to glos.convert named infoOverride

		rRes = glos.read(inputFilename, direct=True)
		self.assertTrue(rRes)

		if infoOverride:
			for key, value in infoOverride.items():
				glos.setInfo(key, value)

		wRes = glos.write(outputFilename, format="Tabfile")
		self.assertEqual(outputFilename, wRes)

		self.compareTextFiles(outputFilename, expectedFilename)
		glos.cleanup()

	def test_convert_csv_txt_4(self):
		self.convert_csv_txt_rw(
			"100-en-fa",
			"100-en-fa",
			infoOverride={"input_file_size": None},
		)


if __name__ == "__main__":
	unittest.main()



import os
import unittest

from glossary_v2_test import TestGlossaryBase


class TestGlossaryDictOrg(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"100-en-fa.dtxt": "05d6e939",
			},
		)
		os.environ["CALC_FILE_SIZE"] = "1"

	def convert_txt_dict_org_source(self, fname, fname2, **convertArgs):
		self.convert(
			f"{fname}.txt",
			f"{fname}-2.dtxt",
			compareText=f"{fname2}.dtxt",
			**convertArgs,
		)

	def test_convert_txt_dict_org_source_1(self):
		self.convert_txt_dict_org_source(
			"100-en-fa",
			"100-en-fa",
		)


if __name__ == "__main__":
	unittest.main()



# -*- coding: utf-8 -*-
# mypy: ignore-errors
#
# Copyright © 2025 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
#
# This program is a free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# You can get a copy of GNU General Public License along this program
# But you can always get it from http://www.gnu.org/licenses/gpl.txt
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

from __future__ import annotations

from os.path import isabs, join
from typing import TYPE_CHECKING, Any

from gi.repository import Gdk as gdk  # noqa: I001
from gi.repository import GLib as glib
from gi.repository import Gtk as gtk

from pyglossary.core import appResDir

if TYPE_CHECKING:
	from collections.abc import Callable

__all__ = [
	"FixedSizePicture",
	"HBox",
	"VBox",
	"getWorkAreaSize",
	"gtk_event_iteration_loop",
	"hasLightTheme",
	"pack",
	"rgba_parse",
	"showInfo",
]


def getWorkAreaSize(_w: object) -> tuple[int, int]:
	display = gdk.Display.get_default()
	# monitor = display.get_monitor_at_surface(w.get_surface())
	# if monitor is None:
	monitor = display.get_primary_monitor()
	rect = monitor.get_workarea()
	return rect.width, rect.height


def gtk_event_iteration_loop() -> None:
	ctx = glib.MainContext.default()
	try:
		while ctx.pending():
			ctx.iteration(True)
	except KeyboardInterrupt:
		pass


def VBox(**kwargs: Any) -> gtk.Box:
	return gtk.Box(orientation=gtk.Orientation.VERTICAL, **kwargs)


def HBox(**kwargs: Any) -> gtk.Box:
	return gtk.Box(orientation=gtk.Orientation.HORIZONTAL, **kwargs)


class FixedSizePicture(gtk.Picture):
	def __init__(self, path: str) -> None:
		gtk.Picture.__init__(self)
		# self.size = size
		if not isabs(path):
			path = join(appResDir, path)
		self.set_filename(path)
		self.set_can_shrink(False)


def imageFromFile(path: str) -> gtk.Image:  # the file must exist
	if not isabs(path):
		path = join(appResDir, path)
	im = gtk.Image()
	im.set_from_file(path)
	return im


def imageFromIconName(iconName: str, size: int, nonStock: bool = False) -> gtk.Image:
	# So gtk.Image.new_from_stock is deprecated
	# And the doc says we should use gtk.Image.new_from_icon_name
	# which does NOT have the same functionality!
	# because not all stock items are existing in all themes (even popular themes)
	# and new_from_icon_name does not seem to look in other (non-default) themes!
	# So for now we use new_from_stock, unless it's not a stock item
	# But we do not use either of these two outside this function
	# So that it's easy to switch
	if nonStock:
		return gtk.Image.new_from_icon_name(iconName)
	try:
		return gtk.Image.new_from_stock(iconName, size)
	except Exception:
		return gtk.Image.new_from_icon_name(iconName)


def rgba_parse(colorStr: str) -> gdk.RGBA:
	rgba = gdk.RGBA()
	if not rgba.parse(colorStr):
		raise ValueError(f"bad color string {colorStr!r}")
	return rgba


def pack(
	box: gtk.Box | gtk.CellLayout,
	child: gtk.Widget | gtk.CellRenderer,
	expand: bool | int = False,
	fill: bool | int = False,  # noqa: ARG001
	padding: int = 0,
) -> None:  # noqa: ARG001
	if padding > 0:
		print(f"pack: padding={padding} ignored")
	if isinstance(box, gtk.Box):
		box.append(child)
		if expand:
			if box.get_orientation() == gtk.Orientation.VERTICAL:
				child.set_vexpand(True)
			else:
				child.set_hexpand(True)
		# FIXME: what to do with: fill, padding
	elif isinstance(box, gtk.CellLayout):
		box.pack_start(child, expand)
	else:
		raise TypeError(f"pack: unknown type {type(box)}")


def newButtonImageBox(label: str, image: gtk.Image, spacing: int = 0) -> gtk.Box:
	hbox = HBox(spacing=spacing)
	pack(hbox, image, 0, 0)
	if label:
		labelObj = gtk.Label(label=label)
		labelObj.set_use_underline(True)
		labelObj.set_xalign(0)
		pack(hbox, labelObj, 0, 0)
	hbox.show()
	return hbox


buttonIconEnable = True
buttonIconSize = 20


def labelIconButton(
	label: str = "",
	iconName: str = "",
	size: gtk.IconSize = gtk.IconSize.NORMAL,
) -> gtk.Button:
	button = gtk.Button()
	if buttonIconEnable:
		button.set_child(
			newButtonImageBox(
				label,
				imageFromIconName(iconName, size),
				spacing=size / 2,
			),
		)
		return button
	button.set_label(label)
	button.set_use_underline(True)
	return button


def labelImageButton(  # noqa: PLR0913
	label: str = "",
	imageName: str = "",
	size: int = 0,
	func: Callable | None = None,
	tooltip: str = "",
	spacing: int = 10,
) -> gtk.Button:
	button = gtk.Button()
	if buttonIconEnable or not label:
		if size == 0:
			size = buttonIconSize
		button.set_child(
			newButtonImageBox(
				label,
				imageFromFile(imageName, size=size),
				spacing=spacing,
			),
		)
		# TODO: the child(HBox) is not centered in the Button
		# problem can be seen in Preferences window: Apply and OK buttons
		# button.set_alignment(0.5, 0.5)
	else:
		button.set_label(label)
		button.set_use_underline(True)
	if func is not None:
		button.connect("clicked", func)
	if tooltip:
		button.set_tooltip_text(tooltip)
	return button


class MyHButtonBox(gtk.Box):
	def __init__(self) -> None:
		gtk.Box.__init__(self, orientation=gtk.Orientation.HORIZONTAL)
		self.get_style_context().add_class("margin_10")
		self._homogeneous = True

	def set_homogeneous(self, homogeneous: bool) -> None:
		# gtk.ButtonBox.set_homogeneous does not work anymore!
		# not sure since when
		self._homogeneous = homogeneous

	def add(self, child: gtk.Widget) -> None:
		self.append(child)  # or prepend?
		if not self._homogeneous:
			self.set_child_non_homogeneous(child, True)
			# New in gi version 3.2.

	def add_button(
		self,
		iconName: str = "",
		imageName: str = "",
		label: str = "",
		onClick: Callable | None = None,
		tooltip: str = "",
	) -> gtk.Button:
		if imageName:
			b = labelImageButton(
				label=label,
				imageName=imageName,
			)
		else:
			b = labelIconButton(
				label,
				iconName,
				gtk.IconSize.NORMAL,
			)
		if onClick:
			b.connect("clicked", onClick)
		if tooltip:
			b.set_tooltip_text(tooltip)
		self.add(b)
		return b

	def add_ok(self, onClick: Callable | None = None) -> gtk.Button:
		return self.add_button(
			# imageName="dialog-ok.svg",
			label="_Confirm",
			onClick=onClick,
		)

	def add_cancel(self, onClick: Callable | None = None) -> gtk.Button:
		return self.add_button(
			# imageName="dialog-cancel.svg",
			label="Cancel",
			onClick=onClick,
		)


def showMsg(  # noqa: PLR0913
	msg: str,
	iconName: str = "",
	parent: gtk.Window | None = None,
	transient_for: gtk.Widget | None = None,
	title: str = "",
	borderWidth: int = 10,  # noqa: ARG001
	iconSize: gtk.IconSize = gtk.IconSize.LARGE,
	selectable: bool = False,
) -> None:
	win = gtk.Window(
		parent=parent,
		transient_for=transient_for,
	)
	# flags=0 makes it skip task bar
	if title:
		win.set_title(title)
	hbox = HBox(spacing=10)
	# hbox.set_border_width(borderWidth)
	if iconName:
		# win.set_icon(...)
		pack(hbox, imageFromIconName(iconName, iconSize))
	label = gtk.Label(label=msg)
	# set_line_wrap(True) makes the window go crazy tall (taller than screen)
	# and that's the reason for label.set_size_request and win.resize
	# label.set_line_wrap(True)
	# label.set_line_wrap_mode(pango.WrapMode.WORD)
	label.set_size_request(500, 1)
	if selectable:
		label.set_selectable(True)
	pack(hbox, label)
	hbox.show()
	vbox = VBox()
	pack(vbox, hbox, 1, 1)

	def onOkClicked(_button: gtk.Button) -> None:
		win.destroy()

	buttonBox = MyHButtonBox()
	buttonBox.add_ok(onClick=onOkClicked)
	hbox2 = HBox()
	pack(hbox2, gtk.Label(), 1, 1)
	pack(hbox2, buttonBox, 0, 0)
	pack(vbox, hbox2, 0, 0)

	win.set_child(vbox)
	# win.resize(600, 1)
	win.show()


def _getLightness(c: gdk.Color) -> float:
	return (max(c.red, c.green, c.blue) + min(c.red, c.green, c.blue)) / 2


def hasLightTheme(widget: gtk.Widget) -> bool:
	fg = widget.get_color()
	# bg = ... no idea how to get it in Gtk4
	# return _getLightness(fg) < _getLightness(bg)
	return _getLightness(fg) < 0.5


def showInfo(msg, **kwargs: Any) -> None:  # noqa: ANN001
	showMsg(msg, iconName="dialog-information", **kwargs)


# def showError(msg, **kwargs) -> None:  # noqa: ANN001
# 	showMsg(msg, iconName="dialog-error", **kwargs)



# -*- coding: utf-8 -*-
#
# Copyright © 2025 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
# This file is part of PyGlossary project, https://github.com/ilius/pyglossary
#
# This program is a free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program. Or on Debian systems, from /usr/share/common-licenses/GPL
# If not, see <http://www.gnu.org/licenses/gpl.txt>.

from __future__ import annotations

from typing import TYPE_CHECKING

from pyglossary.entry_filters import entryFiltersRules
from pyglossary.option import (
	BoolOption,
	FloatOption,
	IntOption,
	StrOption,
)

if TYPE_CHECKING:
	from pyglossary.option import Option

__all__ = ["configDefDict"]

_entryFilterConfigDict = {
	configParam: (filterClass, default)
	for configParam, default, filterClass in entryFiltersRules
	if configParam
}


def getEntryFilterOption(name: str) -> Option:
	filterClass, default = _entryFilterConfigDict[name]
	if isinstance(default, bool):
		optClass = BoolOption
	elif isinstance(default, str):
		optClass = StrOption
	else:
		raise TypeError(f"{default = }")
	return optClass(
		hasFlag=True,
		comment=filterClass.desc,
		falseComment=filterClass.falseComment,
	)


configDefDict: dict[str, Option] = {
	"log_time": BoolOption(
		hasFlag=True,
		comment="Show date and time in logs",
		falseComment="Do not show date and time in logs",
	),
	"cleanup": BoolOption(
		hasFlag=True,
		comment="Cleanup cache or temporary files after conversion",
		falseComment=("Do not cleanup cache or temporary files after conversion",),
	),
	"auto_sqlite": BoolOption(
		hasFlag=False,
		comment=(
			"Auto-enable --sqlite to limit RAM usage when direct\n"
			"mode is not possible. Can override with --no-sqlite"
		),
	),
	"enable_alts": BoolOption(
		hasFlag=True,
		customFlag="alts",
		comment="Enable alternates",
		falseComment="Disable alternates",
	),
	# TODO: version 6.0: replace with "resources"
	# 	comment="Use resources (images, audio, etc)"
	"skip_resources": BoolOption(
		hasFlag=True,
		comment="Skip resources (images, audio, css, etc)",
	),
	"save_info_json": BoolOption(
		hasFlag=True,
		customFlag="info",
		comment="Save .info file alongside output file(s)",
	),
	"lower": getEntryFilterOption("lower"),
	"utf8_check": getEntryFilterOption("utf8_check"),
	"rtl": getEntryFilterOption("rtl"),
	"remove_html": getEntryFilterOption("remove_html"),
	"remove_html_all": getEntryFilterOption("remove_html_all"),
	"normalize_html": getEntryFilterOption("normalize_html"),
	"skip_duplicate_headword": getEntryFilterOption("skip_duplicate_headword"),
	"trim_arabic_diacritics": getEntryFilterOption("trim_arabic_diacritics"),
	"unescape_word_links": getEntryFilterOption("unescape_word_links"),
	"color.enable.cmd.unix": BoolOption(
		hasFlag=False,
		comment="Enable colors in Linux/Unix command line",
	),
	"color.enable.cmd.windows": BoolOption(
		hasFlag=False,
		comment="Enable colors in Windows command line",
	),
	"color.cmd.critical": IntOption(
		hasFlag=False,
		comment="Color code for critical errors in command line",
		minim=0,
		maxim=256,
	),
	"color.cmd.error": IntOption(
		hasFlag=False,
		comment="Color code for errors in command line",
		minim=0,
		maxim=256,
	),
	"color.cmd.warning": IntOption(
		hasFlag=False,
		comment="Color code for warnings in command line",
		minim=0,
		maxim=256,
	),
	# interactive command line interface:
	"cmdi.prompt.indent.str": StrOption(hasFlag=False),
	"cmdi.prompt.indent.color": IntOption(
		hasFlag=False,
		minim=0,
		maxim=256,
	),
	"cmdi.prompt.msg.color": IntOption(
		hasFlag=False,
		minim=0,
		maxim=256,
	),
	"cmdi.msg.color": IntOption(
		hasFlag=False,
		minim=0,
		maxim=256,
	),
	# general GUI options
	"ui_autoSetFormat": BoolOption(hasFlag=False),
	# Tkinter
	"tk.progressbar.color.fill": StrOption(
		hasFlag=False,
		comment="Tkinter: progressbar fill color",
	),
	"tk.progressbar.color.background": StrOption(
		hasFlag=False,
		comment="Tkinter: progressbar background color",
	),
	"tk.progressbar.color.text": StrOption(
		hasFlag=False,
		comment="Tkinter: progressbar text color",
	),
	"tk.progressbar.font": StrOption(
		hasFlag=False,
		comment='Tkinter: progressbar text font. Example: "Sans", "Sans 15"',
	),
	# Reverse
	"reverse_matchWord": BoolOption(hasFlag=False),
	"reverse_showRel": StrOption(hasFlag=False),
	"reverse_saveStep": IntOption(hasFlag=False),
	"reverse_minRel": FloatOption(hasFlag=False),
	"reverse_maxNum": IntOption(hasFlag=False),
	"reverse_includeDefs": BoolOption(hasFlag=False),
}



import hashlib
import os
import unittest

from g_stardict_test import TestGlossaryStarDictBase
from glossary_v2_errors_test import TestGlossaryErrorsBase

from pyglossary.glossary_v2 import Glossary


class TestGlossaryStarDictSortCustom(TestGlossaryStarDictBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryErrorsBase.__init__(self, *args, **kwargs)
		self.dataFileCRC32.update(
			{
				"100-en-fa-sd-v2/100-en-fa.dict": "223a0d1d",
				"100-en-fa-sd-v2/100-en-fa.idx": "6df43378",
				"100-en-fa-sd-v2/100-en-fa.ifo": "bb916827",
				"100-en-fa-sd-v2/100-en-fa.syn": "1160fa0b",
				"100-en-fa-sd-v2.txt": "0b8b2ac0",
			},
		)

	def convert_txt_stardict_enfa(
		self,
		fname,
		**convertArgs,
	):
		self.convert_txt_stardict(
			fname,
			fname + "-sd-v2",
			config={"enable_alts": True},
			info={
				"sourceLang": "English",
				"targetLang": "Persian",
			},
			**convertArgs,
		)

	def convert_txt_stardict_enfa_1(self):
		sortKeyName = "headword"
		self.convert_txt_stardict_enfa(
			"100-en-fa",
			sortKeyName=sortKeyName,
			sqlite=True,
		)
		self.assertLogWarning(
			f"Ignoring user-defined sort order {sortKeyName!r}"
			", and using sortKey function from Stardict plugin",
		)

	def test_convert_txt_stardict_enfa_2(self):
		sortKeyName = "ebook"
		self.convert_txt_stardict_enfa(
			"100-en-fa",
			sortKeyName=sortKeyName,
			sqlite=False,
		)
		self.assertLogWarning(
			f"Ignoring user-defined sort order {sortKeyName!r}"
			", and using sortKey function from Stardict plugin",
		)

	def test_convert_txt_stardict_enfa_3(self):
		sortKeyName = "stardict:en_US.UTF-8"
		self.convert_txt_stardict_enfa(
			"100-en-fa",
			sortKeyName=sortKeyName,
			sqlite=True,
		)
		self.assertLogWarning(
			f"Ignoring user-defined sort order {sortKeyName!r}"
			", and using sortKey function from Stardict plugin",
		)

	def test_convert_txt_stardict_enfa_4(self):
		sortKeyName = "stardict:fa_IR.UTF-8"
		self.convert_txt_stardict_enfa(
			"100-en-fa",
			sortKeyName=sortKeyName,
			sqlite=False,
		)
		self.assertLogWarning(
			f"Ignoring user-defined sort order {sortKeyName!r}"
			", and using sortKey function from Stardict plugin",
		)

	def test_convert_txt_stardict_enfa_5(self):
		os.environ["NO_SQLITE"] = "1"
		self.convert_txt_stardict_enfa("100-en-fa", sqlite=False)
		del os.environ["NO_SQLITE"]

	def test_write_manual(self):
		gloss = Glossary()
		gloss.setInfo("title", "repro")

		gloss.addEntry(gloss.newEntry(["Z", "ZZ"], "Z"))
		gloss.addEntry(gloss.newEntry(["A", "AA"], "A"))

		baseName = "test_write_manual"

		sha1ByExt = {
			"ifo": "7e786a6c95eda8a6a88804ab5c1ea16a3f22132c",
			"idx": "656012a08c8283433ce566150c38ed1d0be9dcdc",
			"dict": "0c9d125b39f8a674cee51c774d4f144fc452ab31",
			"syn": "5349fa18cd1558ff04c18316b412b2692eb45ca2",
		}
		pathByExt = {ext: self.newTempFilePath(f"{baseName}.{ext}") for ext in sha1ByExt}

		gloss.write(pathByExt["ifo"], formatName="Stardict", dictzip=False)

		for ext, sha1sum in sha1ByExt.items():
			with open(pathByExt[ext], mode="rb") as file:
				actualSha1 = hashlib.sha1(file.read()).hexdigest()
				self.assertEqual(sha1sum, actualSha1, f"{ext=}")


if __name__ == "__main__":
	unittest.main()



# -*- coding: utf-8 -*-
# mypy: ignore-errors
#
# Copyright © 2025 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
#
# This program is a free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# You can get a copy of GNU General Public License along this program
# But you can always get it from http://www.gnu.org/licenses/gpl.txt
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

from __future__ import annotations

import logging
from os.path import isfile
from typing import TYPE_CHECKING, Any

from gi.repository import Gdk as gdk
from gi.repository import Gio as gio
from gi.repository import Gtk as gtk
from gi.repository import Pango as pango

from pyglossary.core import homePage
from pyglossary.glossary_v2 import ConvertArgs, Error, Glossary
from pyglossary.os_utils import abspath2
from pyglossary.text_utils import urlToPath
from pyglossary.ui.base import (
	aboutText,
	authors,
	licenseText,
	logo,
)
from pyglossary.ui.ui_gtk4.general_options import GeneralOptionsDialog
from pyglossary.ui.version import getVersion

from .about import AboutWidget
from .console import ConvertConsole
from .file_widgets import InputFileBox, OutputFileBox
from .format_widgets import FormatOptionsDialog, InputFormatBox, OutputFormatBox
from .utils import (
	HBox,
	VBox,
	getWorkAreaSize,
	gtk_event_iteration_loop,
	hasLightTheme,
	pack,
)

if TYPE_CHECKING:
	from collections.abc import Callable

	from pyglossary.config_type import ConfigType
	from pyglossary.ui_type import UIType

__all__ = ["MainWindow"]


log = logging.getLogger("pyglossary")

_ = str  # later replace with translator function


# GTK 4 has removed the GtkContainer::border-width property
# (together with the rest of GtkContainer).
# Use other means to influence the spacing of your containers,
# such as the CSS margin and padding properties on child widgets,
# or the CSS border-spacing property on containers.


class ConvertStatusBox(gtk.Box):
	def __init__(
		self,
		clearClicked: Callable,
	) -> None:
		gtk.Box.__init__(self, orientation=gtk.Orientation.HORIZONTAL, spacing=5)
		self.statusNewId = 0
		# https://gitlab.gnome.org/GNOME/gtk/-/issues/4205
		clearButton = gtk.Button(
			label=_("Clear"),
			# icon_name="gtk-clear",
		)
		clearButton.show()
		clearButton.connect("clicked", clearClicked)
		clearButton.set_tooltip_text("Clear Console")
		pack(self, clearButton)
		####
		# pack(hbox, gtk.Label(), 1, 1)
		######
		self.verbosityLabel = gtk.Label(label=_("Verbosity:"))
		pack(self, self.verbosityLabel)
		##
		self.verbosityCombo = combo = gtk.ComboBoxText()
		for level, levelName in enumerate(log.levelNamesCap):
			combo.append_text(f"{level} - {_(levelName)}")
		combo.set_active(log.getVerbosity())
		combo.connect("changed", self.verbosityComboChanged)
		pack(self, combo)
		####
		# pack(hbox, gtk.Label(), 1, 1)
		####
		self.statusBar = gtk.Statusbar()
		pack(self, self.statusBar, 1, 1)
		####
		# ResizeButton does not work in Gtk 4.0

	def status(self, msg: str) -> None:
		self.statusBar.push(self.statusNewId % 4294967295, msg)
		self.statusNewId += 1

	def verbosityComboChanged(self, _widget: gtk.Widget | None = None) -> None:
		verbosity = self.verbosityCombo.get_active()
		# or int(self.verbosityCombo.get_active_text())
		log.setVerbosity(verbosity)


def getFontSizePixels(widget: gtk.Widget) -> int:
	# Get default font description
	pango_ctx = widget.get_pango_context()
	font_desc = pango_ctx.get_font_description()

	# Pango font size is in Pango units (1/1024 of a point)
	font_size_points = font_desc.get_size() / pango.SCALE

	# Get DPI from monitor (fallback = 96.0)
	display = widget.get_display()
	monitor = display.get_primary_monitor()
	scale = monitor.get_scale_factor()
	dpi = 96.0 * scale  # GTK doesn’t expose real DPI anymore — approximate

	return font_size_points * dpi / 72.0


class MainWindow(gtk.ApplicationWindow):
	# @property
	# def config(self):
	# 	return self.ui.config

	css = """
check {
	min-width: 1.25em;
	min-height: 1.25em;
}

progressbar progress, trough {min-height: 0.6em;}

.margin_03 {
	margin-top: 0.5em;
	margin-right: 0.5em;
	margin-bottom: 0.5em;
	margin-left: 0.5em;
}

.margin_05 {
	margin-top: 0.5em;
	margin-right: 0.5em;
	margin-bottom: 0.5em;
	margin-left: 0.5em;
}

.margin_10 {
	margin-top: 1em;
	margin-right: 1em;
	margin-bottom: 1em;
	margin-left: 1em;
}
	"""

	def status(self, msg: str) -> None:
		self.statusBox.status(msg)

	def __init__(
		self,
		ui: UIType,
		app: gtk.Application,
		progressBar: gtk.ProgressBar | None,
		**kwargs: Any,
	) -> None:
		self.app = app
		self.ui = ui
		#####
		gtk.ApplicationWindow.__init__(self, application=app, **kwargs)
		#####
		self.lightTheme = hasLightTheme(self)
		# this may not be needed in most themes,
		# but just to be sure the colors are not messed up
		if self.lightTheme:
			self.css += "textview.console text {background-color: rgb(255, 255, 255);}"
		else:
			self.css += "textview.console text {background-color: rgb(0, 0, 0);}"
		#####
		self.set_title("PyGlossary (Gtk4)")
		self.progressBar = progressBar
		#####
		self.vbox = VBox()
		self.set_child(self.vbox)
		#####
		screenW, screenH = getWorkAreaSize(self)
		winSize = min(800, screenW - 50, screenH - 50)
		self.set_default_size(winSize, winSize)
		#####
		ckey = gtk.EventControllerKey()
		ckey.connect("key-pressed", self.onKeyPress)
		self.add_controller(ckey)
		####
		self.connect("close-request", self.onCloseRequest)
		####
		self.readOptions = {}
		self.writeOptions = {}
		self.generalOptionsdialog: gtk.Dialog | None = None
		####
		self.pages = []
		self.convertOptions = {}
		#####
		self.styleProvider = gtk.CssProvider()
		gtk.StyleContext.add_provider_for_display(
			gdk.Display.get_default(),
			self.styleProvider,
			gtk.STYLE_PROVIDER_PRIORITY_APPLICATION,
		)
		self.styleProvider.load_from_data(self.css)
		#####
		self.assert_quit = False  # FIXME: not used
		# ____________________ Tab 1 - Convert ____________________ #
		labelSizeGroup = gtk.SizeGroup(mode=gtk.SizeGroupMode.HORIZONTAL)
		buttonSizeGroup = gtk.SizeGroup(mode=gtk.SizeGroupMode.HORIZONTAL)
		self.inOutBoxes: list[gtk.Box] = []
		####
		page = VBox(spacing=5)
		page.label = _("Convert")
		page.icon = ""  # "*.png"
		self.pages.append(page)
		######
		self.inputFileBox = InputFileBox(
			entryChanged=self.inputFileEntryChanged,
			labelSizeGroup=labelSizeGroup,
			buttonSizeGroup=buttonSizeGroup,
		)
		self.inOutBoxes.append(self.inputFileBox)
		pack(page, self.inputFileBox)
		###
		self.inputFormatBox = InputFormatBox(
			self.app,
			parent=self,
			labelSizeGroup=labelSizeGroup,
		)
		self.inOutBoxes.append(self.inputFormatBox)
		pack(page, self.inputFormatBox)
		#####
		hbox = HBox()
		hbox.get_style_context().add_class("margin_03")
		pack(page, hbox)
		#####
		self.outputFileBox = OutputFileBox(
			entryChanged=self.outputFileEntryChanged,
			labelSizeGroup=labelSizeGroup,
			buttonSizeGroup=buttonSizeGroup,
		)
		self.inOutBoxes.append(self.outputFileBox)
		pack(page, self.outputFileBox)
		###
		self.outputFormatBox = OutputFormatBox(
			self.app,
			parent=self,
			labelSizeGroup=labelSizeGroup,
		)
		self.inOutBoxes.append(self.outputFormatBox)
		pack(page, self.outputFormatBox)
		#####
		hbox = HBox(spacing=10)
		hbox.get_style_context().add_class("margin_03")
		pack(hbox, gtk.Label(), expand=True)
		##
		action = gio.SimpleAction.new("readOptions", None)
		action.connect("activate", self.readOptionsClicked)
		app.add_action(action)
		##
		action = gio.SimpleAction.new("writeOptions", None)
		action.connect("activate", self.writeOptionsClicked)
		app.add_action(action)
		##
		action = gio.SimpleAction.new("generalOptions", None)
		action.connect("activate", self.generalOptionsClicked)
		app.add_action(action)
		##
		fontSize = getFontSizePixels(self)
		##
		button = gtk.MenuButton(label=" Options")
		button.set_size_request(0, int(fontSize * 1.8))
		menu = gio.Menu()
		menu.append("Read Options", "app.readOptions")
		menu.append("Write Options", "app.writeOptions")
		menu.append("General Options", "app.generalOptions")
		button.set_menu_model(menu)
		pack(hbox, button)
		##
		button = self.convertButton = gtk.Button()
		button.set_size_request(0, int(fontSize * 1.8))
		button.set_label("    Convert    ")
		button.connect("clicked", self.convertClicked)
		pack(hbox, button)
		##
		self.inOutBoxes.append(hbox)
		pack(page, hbox)
		#####
		self.convertConsole = ConvertConsole(
			self,
			lightTheme=self.lightTheme,
		)
		log.addHandler(self.convertConsole.handler)
		pack(page, self.convertConsole, expand=True)
		######
		self.progressTitle = ""
		if progressBar:
			pack(page, progressBar)
		######
		self.statusBox = ConvertStatusBox(
			clearClicked=self.consoleClearButtonClicked,
		)
		pack(page, self.statusBox)
		# ____________________________________ ____________________ #
		self.pages.append(self.makeAboutWidget())
		pack(self.vbox, self.setupNotebook(self.pages), 1, 1)
		######
		self.vbox.show()
		self.status("Select input file")

	def readOptionsClicked(self, _action: gtk.Action, _param: Any) -> None:
		formatName = self.inputFormatBox.getActive()
		if not formatName:
			log.error("Read Options: input format is not selected")
			return
		if not Glossary.formatsReadOptions[formatName]:
			log.warning(f"Read Options: there is none for {formatName}")
			return
		dialog = FormatOptionsDialog(
			app=self.app,
			formatName=formatName,
			kind="r",
			values=self.readOptions,
			transient_for=self,
		)
		dialog.set_title("Read Options for " + formatName)
		dialog.connect("response", self.readOptionsResponse)
		dialog.present()

	def writeOptionsClicked(self, _action: gtk.Action, _param: Any) -> None:
		formatName = self.outputFormatBox.getActive()
		if not formatName:
			log.error("Write Options: output format is not selected")
			return
		if not Glossary.formatsWriteOptions[formatName]:
			log.warning(f"Write Options: there is none for {formatName}")
			return
		dialog = FormatOptionsDialog(
			app=self.app,
			formatName=formatName,
			kind="w",
			values=self.writeOptions,
			transient_for=self,
		)
		dialog.set_title("Write Options for " + formatName)
		dialog.connect("response", self.writeOptionsResponse)
		dialog.present()

	def readOptionsResponse(
		self,
		dialog: FormatOptionsDialog,
		response_id: gtk.ResponseType,
	) -> None:
		if response_id == gtk.ResponseType.OK:
			self.readOptions = dialog.getOptionsValues()
		dialog.destroy()

	def writeOptionsResponse(
		self,
		dialog: FormatOptionsDialog,
		response_id: gtk.ResponseType,
	) -> None:
		if response_id == gtk.ResponseType.OK:
			self.writeOptions = dialog.getOptionsValues()
		dialog.destroy()

	def generalOptionsClicked(self, _action: gtk.Action, _param: Any) -> None:
		if self.generalOptionsdialog is None:
			self.generalOptionsdialog = GeneralOptionsDialog(self)
		self.generalOptionsdialog.present()

	def makeAboutWidget(self) -> gtk.Widget:
		about = AboutWidget(
			logo=logo,
			header=f"PyGlossary\nVersion {getVersion()}",
			# about=summary,
			about=f'{aboutText}\n<a href="{homePage}">{homePage}</a>',
			authors="\n".join(authors),
			license_text=licenseText,
		)
		about.label = _("About")
		about.icon = ""  # "dialog-information-22.png"
		return about

	def setupNotebook(self, pages: list[gtk.Widget]) -> gtk.Notebook:
		notebook = gtk.Notebook()
		####
		for page in pages:
			tabWidget = gtk.Label(label=page.label, use_underline=True)
			# if vbox.icon:
			# 	tabWidget = VBox(spacing=3)
			# 	pack(tabWidget, imageFromFile(vbox.icon))
			# 	pack(tabWidget, label)
			# 	tabWidget.show()
			notebook.append_page(page, tabWidget)
			try:
				notebook.set_tab_reorderable(page, True)
			except AttributeError:
				pass
		return notebook

	def run(  # noqa: PLR0913
		self,
		inputFilename: str = "",
		outputFilename: str = "",
		inputFormat: str = "",
		outputFormat: str = "",
		reverse: bool = False,
		config: ConfigType | None = None,
		readOptions: dict[str, Any] | None = None,
		writeOptions: dict[str, Any] | None = None,
		convertOptions: dict[str, Any] | None = None,
		glossarySetAttrs: dict[str, Any] | None = None,
	) -> None:
		self.config = config or {}

		if inputFilename:
			self.inputFileBox.set_text(abspath2(inputFilename))
		if outputFilename:
			self.outputFileBox.set_text(abspath2(outputFilename))

		if inputFormat and inputFormat not in Glossary.readFormats:
			log.error(f"invalid {inputFormat=}")
			inputFormat = ""

		if outputFormat and outputFormat not in Glossary.writeFormats:
			log.error(f"invalid {outputFormat=}")
			outputFormat = ""

		if inputFormat:
			self.inputFormatBox.setActive(inputFormat)
		if outputFormat:
			self.outputFormatBox.setActive(outputFormat)

		if reverse:
			log.error("Gtk interface does not support Reverse feature")

		if readOptions:
			self.readOptions = readOptions
		if writeOptions:
			self.writeOptions = writeOptions

		self.convertOptions = convertOptions
		if convertOptions:
			log.debug(f"Using {convertOptions=}")

		self._glossarySetAttrs = glossarySetAttrs or {}
		self.present()

	def exitApp(self) -> None:
		self.destroy()
		# unlike Gtk3, no need for sys.exit or gtk.main_quit (which does not exist)

	def onCloseRequest(self, _widget: gtk.Widget) -> None:
		self.exitApp()

	def onKeyPress(
		self,
		_ckey: gtk.EventControllerKey,
		keyval: int,
		_keycode: int,
		_state: gdk.ModifierType,
	) -> None:
		if keyval == gdk.KEY_Escape:
			self.exitApp()

	def onButtonPress(
		self,
		gesture: gtk.GestureClick,
		_n_press: int,
		_x: int,
		_y: int,
	) -> None:
		print(f"MainWindow.onButtonPress: {gesture}")

	def consoleClearButtonClicked(self, _widget: gtk.Widget = None) -> None:
		self.convertConsole.set_text("")

	def setInOutBoxesSensivive(self, sensitive: bool) -> None:
		for hbox in self.inOutBoxes:
			hbox.set_sensitive(sensitive)

	def convertClicked(self, _widget: gtk.Widget | None = None) -> None:
		inPath = self.inputFileBox.get_text()
		if not inPath:
			log.critical("Input file path is empty!")
			return
		inFormat = self.inputFormatBox.getActive()

		outPath = self.outputFileBox.get_text()
		if not outPath:
			log.critical("Output file path is empty!")
			return
		outFormat = self.outputFormatBox.getActive()

		self.status("Converting...")
		self.setInOutBoxesSensivive(False)
		gtk_event_iteration_loop()

		self.progressTitle = "Converting"
		readOptions = self.readOptions
		writeOptions = self.writeOptions
		glos = Glossary(ui=self.ui)
		glos.config = self.config
		glos.progressbar = self.progressBar is not None

		for attr, value in self._glossarySetAttrs.items():
			setattr(glos, attr, value)

		log.debug(f"readOptions: {readOptions}")
		log.debug(f"writeOptions: {writeOptions}")
		log.debug(f"convertOptions: {self.convertOptions}")
		log.debug(f"config: {self.config}")

		try:
			glos.convert(
				ConvertArgs(
					inPath,
					inputFormat=inFormat,
					outputFilename=outPath,
					outputFormat=outFormat,
					readOptions=readOptions,
					writeOptions=writeOptions,
					**self.convertOptions,
				),
			)
			self.status("Convert finished")

		except Error as e:
			log.critical(str(e))
			glos.cleanup()

		finally:
			self.setInOutBoxesSensivive(True)
			self.assert_quit = False
			self.progressTitle = ""

	def inputFileEntryChanged(self, entry: gtk.Entry) -> None:
		inPath = entry.get_text()
		inFormat = self.inputFormatBox.getActive()
		if inPath.startswith("file://"):
			inPath = urlToPath(inPath)
			entry.set_text(inPath)

		if self.config["ui_autoSetFormat"] and not inFormat:
			try:
				inputArgs = Glossary.detectInputFormat(inPath)
			except Error:
				pass
			else:
				self.inputFormatBox.setActive(inputArgs.formatName)

		if not isfile(inPath):
			return

		self.status("Select output file")

	def outputFileEntryChanged(self, entry: gtk.Entry) -> None:
		outPath = entry.get_text()
		outFormat = self.outputFormatBox.getActive()
		if not outPath:
			return
		if outPath.startswith("file://"):
			outPath = urlToPath(outPath)
			entry.set_text(outPath)

		if self.config["ui_autoSetFormat"] and not outFormat:
			try:
				outputArgs = Glossary.detectOutputFormat(
					filename=outPath,
					inputFilename=self.inputFileBox.get_text(),
				)
			except Error:
				pass
			else:
				outFormat = outputArgs.formatName
				self.outputFormatBox.setActive(outFormat)

		if outFormat:
			self.status('Press "Convert"')
		else:
			self.status("Select output format")



# -*- coding: utf-8 -*-
# mypy: ignore-errors
#
# Copyright © 2025 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
#
# This program is a free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# You can get a copy of GNU General Public License along this program
# But you can always get it from http://www.gnu.org/licenses/gpl.txt
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

from __future__ import annotations

import logging
import traceback
from typing import TYPE_CHECKING, Protocol

from gi.repository import Gtk as gtk

if TYPE_CHECKING:
	from gi.repository import Gdk as gdk

__all__ = ["GtkSingleTextviewLogHandler", "MainWinType"]

log = logging.getLogger("pyglossary")


class MainWinType(Protocol):
	def status(self, msg: str) -> None: ...


class GtkTextviewLogHandler(logging.Handler):
	def __init__(
		self,
		mainWin: MainWinType,
		textview_dict: dict[str, gtk.TextView],
	) -> None:
		logging.Handler.__init__(self)

		self.mainWin = mainWin
		self.buffers = {}
		for levelNameCap in log.levelNamesCap[:-1]:
			levelName = levelNameCap.upper()
			textview = textview_dict[levelName]

			buff = textview.get_buffer()
			tag = gtk.TextTag.new(levelName)
			buff.get_tag_table().add(tag)

			self.buffers[levelName] = buff

	def getTag(self, levelname: str) -> gtk.TextTag:
		return self.buffers[levelname].get_tag_table().lookup(levelname)

	def setColor(self, levelname: str, rgba: gdk.RGBA) -> None:
		self.getTag(levelname).set_property("foreground-rgba", rgba)
		# foreground-gdk is deprecated since Gtk 3.4

	def emit(self, record: logging.LogRecord) -> None:
		msg = ""
		if record.getMessage():
			msg = self.format(record)
		# msg = msg.replace("\x00", "")

		if record.exc_info:
			type_, value, tback = record.exc_info
			tback_text = "".join(
				traceback.format_exception(type_, value, tback),
			)
			if msg:
				msg += "\n"
			msg += tback_text

		buff = self.buffers[record.levelname]

		buff.insert_with_tags_by_name(
			buff.get_end_iter(),
			msg + "\n",
			record.levelname,
		)

		if record.levelno == logging.CRITICAL:
			self.mainWin.status(record.getMessage())


class GtkSingleTextviewLogHandler(GtkTextviewLogHandler):
	def __init__(self, mainWin: MainWinType, textview: gtk.TextView) -> None:
		GtkTextviewLogHandler.__init__(
			self,
			mainWin,
			{
				"CRITICAL": textview,
				"ERROR": textview,
				"WARNING": textview,
				"INFO": textview,
				"DEBUG": textview,
				"TRACE": textview,
			},
		)



import datetime
import hashlib
import sys
import unittest
from os.path import abspath, dirname

from freezegun import freeze_time

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

from glossary_v2_test import TestGlossaryBase

from pyglossary.glossary_v2 import ConvertArgs, Glossary

testTimeEpoch = 1730579400
testTime = datetime.datetime.fromtimestamp(testTimeEpoch, tz=datetime.UTC)


class TestGlossaryYomichan(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"050-JMdict-English-v3.txt": "6068b9a7",
			},
		)

	@freeze_time(testTime)
	def convert_to_yomichan(
		self,
		inputFname,
		testId,
		sha1sum="",
		**convertArgs,
	):
		inputFilename = self.downloadFile(inputFname)
		outputFilename = self.newTempFilePath(
			f"{inputFname.replace('.', '_')}-{testId}.zip",
		)

		glos = self.glos = Glossary()
		res = glos.convert(
			ConvertArgs(
				inputFilename=inputFilename,
				outputFilename=outputFilename,
				outputFormat="Yomichan",
				**convertArgs,
			)
		)
		self.assertEqual(outputFilename, res)

		if sha1sum:
			with open(outputFilename, mode="rb") as file:
				actualSha1 = hashlib.sha1(file.read()).hexdigest()
			self.assertEqual(sha1sum, actualSha1, f"{outputFilename=}")

	def test_convert_txt_yomichan_1(self):
		if sys.version_info[:2] == (3, 13):
			self.skipTest("Skipping test on this Python version")
		self.convert_to_yomichan(
			"050-JMdict-English-v3.txt",
			testId="1",
			# sha1sum="e54bc12755924586c306831b54a44a3dfd45cf7b",  # FIXME
		)

	def test_convert_txt_yomichan_2(self):
		if sys.version_info[:2] == (3, 13):
			self.skipTest("Skipping test on this Python version")
		self.convert_to_yomichan(
			"100-ja-en.txt",
			testId="2",
			# sha1sum="02bf6195eba15d0e76b3b119fa9c57d3f17eb169",  # FIXME
		)


if __name__ == "__main__":
	unittest.main()



from __future__ import annotations

import sys
import unittest
from os.path import abspath, dirname
from typing import TYPE_CHECKING

from glossary_v2_test import TestGlossaryBase

if TYPE_CHECKING:
	pass

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)


class TestGlossaryDictfile(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"edict2/024-cedict.u8": "de30cdd3",
				"edict2/024-cedict.txt": "34e2dc56",
				"edict2/024-cedict-links.txt": "74c3446f",
				"edict2/024-cedict-nocolor.txt": "dafb8cd8",
				"edict2/024-cedict-trad.txt": "eec67f1e",
				"edict2/024-cedict-v2.txt": "1c203fa2",
				"edict2/024-cedict-v2-links.txt": "89d9376a",
				"edict2/024-cedict-v2-nocolor.txt": "9710ebd7",
				"edict2/024-cedict-v2-trad.txt": "5798f4cc",
			},
		)

	def convert_edict_txt(self, fname, fname2):
		self.convert(
			f"edict2/{fname}.u8",
			f"{fname}-2.txt",
			compareText=f"edict2/{fname2}.txt",
			name=f"{fname}.u8",
			readOptions={
				"summary_alternatives": True,
			},
		)

	def test_convert_edict_txt_1(self):
		self.convert(
			"edict2/024-cedict.u8",
			"024-cedict-1a.txt",
			compareText="edict2/024-cedict-v2.txt",
			name="024-cedict.u8",
		)

	def test_convert_edict_txt_1_old(self):
		self.convert(
			"edict2/024-cedict.u8",
			"024-cedict-1b.txt",
			compareText="edict2/024-cedict.txt",
			name="024-cedict.u8",
			readOptions={
				"summary_alternatives": True,
			},
		)

	def test_convert_edict_txt_2(self):
		self.convert(
			"edict2/024-cedict.u8",
			"024-cedict-2a.txt",
			compareText="edict2/024-cedict-v2-trad.txt",
			name="024-cedict.u8",
			readOptions={
				"traditional_title": True,
			},
		)

	def test_convert_edict_txt_2_old(self):
		self.convert(
			"edict2/024-cedict.u8",
			"024-cedict-2b.txt",
			compareText="edict2/024-cedict-trad.txt",
			name="024-cedict.u8",
			readOptions={
				"traditional_title": True,
				"summary_alternatives": True,
			},
		)

	def test_convert_edict_txt_3(self):
		self.convert(
			"edict2/024-cedict.u8",
			"024-cedict-3a.txt",
			compareText="edict2/024-cedict-v2-nocolor.txt",
			name="024-cedict.u8",
			readOptions={
				"colorize_tones": False,
			},
		)

	def test_convert_edict_txt_3_old(self):
		self.convert(
			"edict2/024-cedict.u8",
			"024-cedict-3b.txt",
			compareText="edict2/024-cedict-nocolor.txt",
			name="024-cedict.u8",
			readOptions={
				"colorize_tones": False,
				"summary_alternatives": True,
			},
		)

	def test_convert_edict_txt_4(self):
		self.convert(
			"edict2/024-cedict.u8",
			"024-cedict-4a.txt",
			compareText="edict2/024-cedict-v2-links.txt",
			name="024-cedict.u8",
			readOptions={
				"link_references": True,
			},
		)

	def test_convert_edict_txt_4_old(self):
		self.convert(
			"edict2/024-cedict.u8",
			"024-cedict-4b.txt",
			compareText="edict2/024-cedict-links.txt",
			name="024-cedict.u8",
			readOptions={
				"link_references": True,
				"summary_alternatives": True,
			},
		)


if __name__ == "__main__":
	unittest.main()



import gzip
import os
import unittest

from glossary_v2_test import TestGlossaryBase


class TestGlossaryKobo(TestGlossaryBase):
	def setUp(self):
		if os.getenv("SKIP_TEST_MISSING_DEP"):
			try:
				import marisa_trie  # noqa: F401
			except ImportError:
				self.skipTest("skipping module due to missing dependency: marisa_trie")
		TestGlossaryBase.setUp(self)

	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)
		# self.dataFileCRC32.update({})

	def convert_txt_kobo(self, fname, sha1sumDict, **convertArgs):
		outputFname = f"{fname}-2.kobo.zip"
		outputFpath = self.newTempFilePath(outputFname)
		# expectedFpath = self.downloadFile(f"{fname}.kobo.zip")
		self.convert(
			f"{fname}.txt",
			outputFname,
			**convertArgs,
		)
		dataReplaceFuncs = {
			zfname: gzip.decompress for zfname in sha1sumDict if zfname != "words"
		}
		self.checkZipFileSha1sum(
			outputFpath,
			sha1sumDict=sha1sumDict,
			dataReplaceFuncs=dataReplaceFuncs,
		)

	def test_convert_txt_kobo_1(self):
		sha1sumDict = {
			"11.html": "39f0f46560da7398ab0d3b19cc1c2387ecd201dd",
			"aa.html": "df9460450e8b46e913c57bf39dcc799ffdc2fb33",
			"ab.html": "be4271a8508dbb499bafd439810af621a7b3474f",
			"words": "d0f74e854f090fbaa8211bcfd162ad99ec4da0a3",
		}
		self.convert_txt_kobo("100-en-fa", sha1sumDict)


if __name__ == "__main__":
	unittest.main()
